/**
 * Implement iOS native API for HDP or generic web view, depending on param passed to .initialize after evaluation
 */

const ZMOB_nativeAPI = (typeof module !== "undefined" && module.exports) || {};
(function (api) {
  var i = null, exports = api;
 
  var featuresMap = {
    "IMPROVED_REQUEST_HANDLING" : true
  };

  function loadURI(uri) {
    var iframe = document.createElement("IFRAME");
    iframe.setAttribute("src", uri);
    document.documentElement.appendChild(iframe);
    iframe.parentNode.removeChild(iframe);
  }
  
  function invokeMethodWithArguments(uri) {
    var args = Array.prototype.slice.call(arguments, 1);
    if (args.length) {
      uri += "/" + args.join("/");
    }
    loadURI(uri);
  }
  
  function invokeMethod(methodname) {
    var args = Array.prototype.slice.call(arguments, 1),
        uri = "/js-to-objc-message/" + methodname;
    args.unshift(uri);
    invokeMethodWithArguments.apply(null, args);
  }

  const HDP_API = {
    /**
     * TODO Implement iOS control of the photo gallery view.
     */
    displayPhotoViewer: function (Xoffset, Yoffset, width, height) {
        invokeMethod('hdpPhotoGalleryBox', Xoffset, Yoffset, width, height);
    },
    /**
     * These methods are defined for the API, but are not implement in iOS applications.
    saveFavoriteHome: function () {}, 
    removeFavoriteHome: function () {},
    launchStreetView: function () {},
    trackFiksuContactEvent: function (category, event, label) {},
    setStreetViewAngle: function (status, yaw) {},
    **/
    viewOnMap: function () {
      invokeMethod('view-on-map');
    },
    emailAgent: function (email, subject) {
      alert ("The emailAgent code isn't implemented in the iOS controller, yet! Get John to fix it!");
    },
    callAgent: function (phone) {
      invokeMethod('tel', phone);
    },
    launchAmenities: function () {
      invokeMethod('show-amenities');
    },
    launchDirections: function () {
      invokeMethod('driving-directions');
    },
    setMapRectangle: function (Xoffset, Yoffset, width, height) {
      invokeMethod('hdpMapBox', Xoffset, Yoffset, width, height);
    },
    setContactModuleRectangle: function (Xoffset, Yoffset, width, height) {
      invokeMethod('contactModuleRect', Xoffset, Yoffset, width, height);
    },
    launchPaymentCalculator: function () {
      invokeMethod('payment-calculator');
    },
    launchMortgageRates: function () {
      invokeMethod('current-rates');
    },
    displayForeclosureDisclaimer: function () {
     invokeMethod('legal-disclaimer');
    },
    addToCalendar: function (index) {
      invokeMethod('open-house', index);
    },
    showAddressDialog: function(x, y) {
      invokeMethod('show-address-dialog', x, y);
    },
    echo:function (message) {
      invokeMethod('echo', message);
    },
    logConsoleMessage: function(message, level, emitter) {
      invokeMethodWithArguments('zillow://console.log', new Date().getTime(), message, level, emitter);
    },
    setContactFormJson: function(jsonString) {
      invokeMethod('contact-form-json', jsonString);
    }
  };
 
  const COMMON_API = {
    displayHomeDetails: function (zpid) {
      invokeMethod('home-details', zpid);
    },
    displayAboutZestimate: function () {
      invokeMethod('about-zestimate');
    },
    supportsFeature: function (featureName) {
      return featuresMap[featureName] ? true : false;
    },
    trackWebviewEvent: function (event, action, label, value) {
      invokeMethodWithArguments('zillow://analytics/event', event, action, label, value);
    },
    trackWebviewPageView: function (pageName) {
      invokeMethodWithArguments('zillow://analytics/page', pageName);
    },
    trackCustomVar: function (slot, value) {
      invokeMethodWithArguments('zillow://analytics/setCV', slot, value);
    },
    launchLogin: function (register, reason, analyticsLabel) {
      invokeMethod('auth-reg-signin', register ? true : false, reason, analyticsLabel);
    },
    userLoggedIn: function (emailAddress, zuid, reason) {
      invokeMethod('user-loggedin', emailAddress, zuid, reason);
    },
    showModalAlert:function(title, text) {
      invokeMethod('show-alert', title, text);
    },
  };

  exports.initialize = function (mode) {
   delete exports.initialize;
   for (i in COMMON_API) {
     if (COMMON_API.hasOwnProperty(i)) {
       exports[i] = COMMON_API[i];
     }
   }
   if ('HDP' === mode) {
     for (i in HDP_API) {
       if (HDP_API.hasOwnProperty(i)) {
         exports[i] = HDP_API[i];
       }
     }
   }
 };
})(ZMOB_nativeAPI);
